import { Module, Provider } from '@nestjs/common';
import { CreateAuthenticationGrpcController } from './interface/grpc-controllers/create-authentication.grpc.controller';
import { CreateAuthenticationService } from './core/application/commands/create-authentication/create-authentication.service';
import { AuthenticationMapper } from './authentication.mapper';
import { AUTHENTICATION_REPOSITORY } from './authentication.di-tokens';
import { AuthenticationRepository } from './infrastructure/authentication.repository';
import { PrismaService } from './infrastructure/prisma.service';
import { CqrsModule } from '@nestjs/cqrs';

const grpcControllers = [CreateAuthenticationGrpcController];

const commandHandlers: Provider[] = [CreateAuthenticationService];

const mappers: Provider[] = [AuthenticationMapper];

const repositories: Provider[] = [
  {
    provide: AUTHENTICATION_REPOSITORY,
    useClass: AuthenticationRepository,
  },
];

const orms: Provider[] = [PrismaService];

@Module({
  imports: [CqrsModule],
  controllers: [...grpcControllers],
  providers: [...commandHandlers, ...mappers, ...repositories, ...orms],
  exports: [PrismaService, AuthenticationMapper, AUTHENTICATION_REPOSITORY],
})
export class AuthenticationModule {}
