import { AggregateRoot, AggregateID } from '@mobicoop/ddd-library';
import { v4 } from 'uuid';
import * as bcrypt from 'bcrypt';
import {
  AuthenticationProps,
  CreateAuthenticationProps,
} from './authentication.types';
import { AuthenticationCreatedDomainEvent } from './events/authentication-created.domain-events';

export class AuthenticationEntity extends AggregateRoot<AuthenticationProps> {
  protected readonly _id: AggregateID;

  static create = async (
    create: CreateAuthenticationProps,
  ): Promise<AuthenticationEntity> => {
    const id = v4();
    const props: AuthenticationProps = { ...create };
    const hash = await bcrypt.hash(props.password, 10);
    const authentication = new AuthenticationEntity({
      id,
      props: {
        password: hash,
        usernames: props.usernames,
      },
    });
    authentication.addEvent(
      new AuthenticationCreatedDomainEvent({ aggregateId: id }),
    );
    return authentication;
  };

  validate(): void {
    // entity business rules validation to protect it's invariant before saving entity to a database
  }
}
