import { Injectable, Logger } from '@nestjs/common';
import { EventEmitter2 } from '@nestjs/event-emitter';
import { PrismaRepositoryBase } from '@mobicoop/ddd-library';
import { AuthenticationEntity } from '../core/domain/authentication.entity';
import { AuthenticationRepositoryPort } from '../core/application/commands/ports/authentication.repository.port';
import { PrismaService } from './prisma.service';
import { AuthenticationMapper } from '../authentication.mapper';

export type AuthenticationBaseModel = {
  uuid: string;
  password: string;
  createdAt: Date;
  updatedAt: Date;
};

export type AuthenticationReadModel = AuthenticationBaseModel & {
  usernames: UsernameModel[];
};

export type AuthenticationWriteModel = AuthenticationBaseModel & {
  usernames: {
    create: UsernameModel[];
  };
};

export type UsernameModel = {
  username: string;
  type: string;
};

/**
 *  Repository is used for retrieving/saving domain entities
 * */
@Injectable()
export class AuthenticationRepository
  extends PrismaRepositoryBase<
    AuthenticationEntity,
    AuthenticationReadModel,
    AuthenticationWriteModel
  >
  implements AuthenticationRepositoryPort
{
  constructor(
    prisma: PrismaService,
    mapper: AuthenticationMapper,
    eventEmitter: EventEmitter2,
  ) {
    super(
      prisma.auth,
      prisma,
      mapper,
      eventEmitter,
      new Logger(AuthenticationRepository.name),
    );
  }
}
